# Bio-ensemblrest
# 
# Copyright (C) 2015
#
# author: Natapol Pornputtapong <natapol.por@gmail.com>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#

# raise "Please, use ruby 1.9.0 or later." if RUBY_VERSION < "1.9.0"

module EnsemblREST
	
	@@url = URI.parse('http://rest.ensembl.org')
	@@http_connect = Net::HTTP.new(@@url.host, @@url.port)
	
	SUPPOTEDFORMATS = {
		'text' => 'text/plain',
		'fasta' => 'text/x-fasta',
		'gff3' => 'ext/x-gff3',
		'json' => 'application/json',
		'msgpack' => 'application/x-msgpack',
		'nh' => 'text/x-nh',
		'seqxml' => 'text/x-seqxml+xml', 
		'sereal' => 'application/x-sereal',
		'phyloxml' => 'text/x-phyloxml+xml',
		'xml' => 'text/xml',
		'yaml' => 'text/x-yaml'
	}
	
	class InvalidResponse < Exception
		
	end
	
	class ServiceNotFound < Exception
		
	end
	
	module_function
	
	# Connect to Ensembl REST Service
	# 
	# @param [String] server Base Ensembl REST Service URL
	def connect(server)
		@@url = URI.parse(server)
		@@http_connect = Net::HTTP.new(@@url.host, @@url.port)
	end
	
	# Switch to use GRCh37 assembly version
	def use_GRCh37
		@@url = URI.parse('http://grch37.rest.ensembl.org/')
		@@http_connect = Net::HTTP.new(@@url.host, @@url.port)
	end
	
	# Switch to use GRCh37 assembly version
	def use37
		EnsemblREST::use_GRCh37
	end
	
	# Switch to use GRCh38 assembly version
	def use_GRCh38
		@@url = URI.parse('http://rest.ensembl.org/')
		@@http_connect = Net::HTTP.new(@@url.host, @@url.port)
	end
	
	# Switch to use GRCh38 assembly version
	def use38
		EnsemblREST::use_GRCh38
	end
	
	# Parse options use in the Module only
	def parse_options(options)
		parsed_options = {opts: [], format: {}}
		options.each do |k, v|
			if k.to_s == 'format'
				parsed_options[:format] = {'content-type' => EnsemblREST::SUPPOTEDFORMATS.has_key?(v) ? EnsemblREST::SUPPOTEDFORMATS[v] : v}
			elsif k.to_s == 'content-type'
				parsed_options[:format] = {'content-type' => v}
			else
				if v.is_a?(TrueClass)
					parsed_options[:opts].push("#{k}=1")
				elsif v.is_a?(FalseClass)
					parsed_options[:opts].push("#{k}=0")
				elsif v.is_a?(Array)
					v.each {|member| parsed_options[:opts].push("#{k}=#{member}")}
				else
					parsed_options[:opts].push("#{k}=#{v}")
				end
			end
			
		end
		return parsed_options
	end
	
	# Parse responses use in the Module only
	def parse_response(response, type)
		case response.code
		when '200'
			case type
			when 'application/json'
				return JSON.parse(response.body)
			when 'ext/x-gff3'
				return Bio::GFF::GFF3.new(response.body)
			when 'text/x-fasta'
				return Bio::FastaFormat.new(response.body)
			when 'text/x-phyloxml+xml'
				return Bio::PhyloXML::Parser.new(response.body)
			else
				return response.body
			end
		when '404'
			raise EnsemblREST::ServiceNotFound, "Service not found"
		else
			raise EnsemblREST::InvalidResponse, "Invalid response: #{response.code}"
		end
	end
	
	# Do a get request
	def get(get_path, options)
		parsed_options = EnsemblREST::parse_options(options)
		request = Net::HTTP::Get.new("#{get_path}#{parsed_options[:opts].empty? ? '' : "?#{parsed_options[:opts].join(';')}"}", parsed_options[:format])
		return EnsemblREST::parse_response(@@http_connect.request(request), parsed_options[:format]['content-type'])
	end
	
	# Do a post request
	def post(post_path, request_body, options)
		parsed_options = EnsemblREST::parse_options(options)
		request = Net::HTTP::Post.new(post_path, parsed_options[:format].merge({'Accept' => 'application/json'}))
		request.body = request_body.to_json
		return EnsemblREST::parse_response(@@http_connect.request(request), parsed_options[:format]['content-type'])
	end
end