# Bio-ensemblrest
# 
# Copyright (C) 2015
#
# author: Natapol Pornputtapong <natapol.por@gmail.com>
#
# Documentation: Natapol Pornputtapong (RDoc'd and embellished by William Webber)
#

# raise "Please, use ruby 1.9.0 or later." if RUBY_VERSION < "1.9.0"

module EnsemblREST
	
	module Info
		module_function
		
		# List the names of analyses involved in generating Ensembl data.
		# 
		# @param [String] species Species name/alias
		# @param [Hash] options Optional arguments for the service please goto http://rest.ensembl.org/documentation/info/analysis
		# @return [JSON] List the names of analyses
		def analysis(species = 'human', options = {})
			return EnsemblREST.get("info/analysis/#{species}", {format: 'json'}.merge(options))
		end
		
		# List the currently available assemblies for a species.
		# 
		# @param [String] species Species name/alias
		# @param [Hash] options Optional arguments for the service please goto http://rest.ensembl.org/documentation/info/assembly_info
		# @return [JSON] List the currently available assemblies
		def assembly_info(species = 'human', options = {})
			return EnsemblREST.get("info/assembly/#{species}", {format: 'json'}.merge(options))
		end
		
		# Returns information about the specified toplevel sequence region for the given species.
		# 
		# @param [String] region_name The (top level) sequence region name.
		# @param [String] species Species name/alias
		# @param [Hash] options Optional arguments for the service please goto http://rest.ensembl.org/documentation/info/assembly_stats
		# @return [JSON] Returns information about the specified toplevel sequence region
		def assembly_stats(region_name, species = 'human', options = {})
			return EnsemblREST.get("info/assembly/#{species}/#{region_name}", {format: 'json'}.merge(options))
		end
		
		# List the functional classifications of gene models that Ensembl associates with a particular species. 
		# Useful for restricting the type of genes/transcripts retrieved by other endpoints.
		# 
		# @param [String] species Species name/alias
		# @param [Hash] options Optional arguments for the service please goto http://rest.ensembl.org/documentation/info/biotypes
		# @return [JSON] List the currently available assemblies
		def biotypes(species = 'human', options = {})
			return EnsemblREST.get("info/biotypes/#{species}", {format: 'json'}.merge(options))
		end
		
		# List all compara analyses available (an analysis defines the type of comparative data).
		# 
		# @param [Hash] options Optional arguments for the service please goto http://rest.ensembl.org/documentation/info/compara_methods
		# @return [JSON] List all compara analyses
		def compara_methods(options = {})
			return EnsemblREST.get("info/compara/methods", {format: 'json'}.merge(options))
		end
		
		# List all collections of species analysed with the specified compara method.
		# 
		# @param [String] method Filter by compara method. Use one the methods returned by /info/compara/methods endpoint.
		# @param [Hash] options Optional arguments for the service please goto http://rest.ensembl.org/documentation/info/compara_species_sets
		# @return [JSON] List the currently available assemblies
		def compara_species_sets(method, options = {})
			return EnsemblREST.get("info/compara/species_sets/#{method}", {format: 'json'}.merge(options))
		end
		
		# Lists all available comparative genomics databases and their data release.
		# 
		# @param [Hash] options Optional arguments for the service please goto http://rest.ensembl.org/documentation/info/comparas
		# @return [JSON] List all compara analyses
		def comparas(options = {})
			return EnsemblREST.get("info/comparas", {format: 'json'}.merge(options))
		end
		
		# Shows the data releases available on this REST server. May return more than one release (unfrequent non-standard Ensembl configuration).
		# 
		# @param [Hash] options Optional arguments for the service please goto http://rest.ensembl.org/documentation/info/data
		# @return [JSON] Shows the data releases available on this REST server
		def data(options = {})
			return EnsemblREST.get("info/data", {format: 'json'}.merge(options))
		end
		
		# Lists all available external sources for a species.
		# 
		# @param [String] species Species name/alias
		# @param [Hash] options Optional arguments for the service please goto http://rest.ensembl.org/documentation/info/external_dbs
		# @return [JSON] Lists all available external sources
		def external_dbs(species = 'human', options = {})
			return EnsemblREST.get("info/external_dbs/#{species}", {format: 'json'}.merge(options))
		end
		
		# Checks if the service is alive.
		# 
		# @param [Hash] options Optional arguments for the service please goto http://rest.ensembl.org/documentation/info/ping
		# @return [Boolean] True/False
		def ping(options = {})
			begin
				EnsemblREST.get("info/ping", {format: 'json'}.merge(options))
				return true
			rescue  EnsemblREST::ServiceNotFound
				return false
			end
		end
		
		# Shows the current version of the Ensembl REST API.
		# 
		# @param [Hash] options Optional arguments for the service please goto http://rest.ensembl.org/documentation/info/rest
		# @return [String] version of this REST server
		def rest(options = {})
			return EnsemblREST.get("info/rest", {format: 'json'}.merge(options))["release"]
		end
		
		# Shows the current version of the Ensembl API used by the REST server.
		# 
		# @param [Hash] options Optional arguments for the service please goto http://rest.ensembl.org/documentation/info/software
		# @return [String] the current version of the Ensembl API
		def software(options = {})
			return EnsemblREST.get("info/software", {format: 'json'}.merge(options))["release"]
		end
		
		# Lists all available species, their aliases, available adaptor groups and data release.
		# 
		# @param [Hash] options Optional arguments for the service please goto http://rest.ensembl.org/documentation/info/species
		# @return [String] Lists all available species
		def species(options = {})
			return EnsemblREST.get("info/species", {format: 'json'}.merge(options))
		end
	end
end